import socket
from PyQt5.QtCore import QThread, pyqtSignal

class PhoneClient(QThread):
    data_received = pyqtSignal(bytes, int)  # Include client_id
    state_changed = pyqtSignal(str, str, int)  # Include client_id

    def __init__(self, host, port, client_id):
        super().__init__()
        self.host = host
        self.port = port
        self.client_id = client_id
        self.sock = None
        self.running = True

    def run(self):
        try:
            self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            self.sock.setsockopt(socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1)
            self.sock.settimeout(15)
            self.sock.connect((self.host, self.port))
            print(f"Client {self.client_id} connected to {self.host}:{self.port}")
            while self.running:
                try:
                    data = self.sock.recv(1024)
                    if not data:
                        print(f"Client {self.client_id} disconnected")
                        self.state_changed.emit("CALL_END", "", self.client_id)
                        break
                    decoded_data = data.decode('utf-8', errors='ignore').strip()
                    print(f"Client {self.client_id} received raw: {decoded_data}")
                    if decoded_data in ["RINGING", "CALL_END", "CALL_DROPPED", "IN_CALL"]:
                        self.state_changed.emit(decoded_data, "", self.client_id)
                    else:
                        self.data_received.emit(data, self.client_id)
                        print(f"Client {self.client_id} received audio: {decoded_data}")
                except socket.timeout:
                    print(f"Client {self.client_id} timed out waiting for data")
                    continue
                except Exception as e:
                    print(f"Client {self.client_id} error: {e}")
                    self.state_changed.emit("CALL_END", "", self.client_id)
                    break
        except Exception as e:
            print(f"Client {self.client_id} connection failed: {e}")
        finally:
            if self.sock:
                self.sock.close()

    def send(self, message):
        if self.sock and self.running:
            try:
                self.sock.send(message.encode())
                print(f"Client {self.client_id} sent: {message}")
            except Exception as e:
                print(f"Client {self.client_id} send error: {e}")

    def stop(self):
        self.running = False
        if self.sock:
            self.sock.close()